@@ -1,24 +1,32 @@
+import json
 import yaml
 import click
 
+from proxygen_cli.lib import output
 from proxygen_cli.lib.credentials import CREDENTIALS, _yaml_credentials_file_source
 from proxygen_cli.lib.dot_proxygen import credentials_file
 
+CHOICE_OF_CREDENTIAL_KEYS = click.Choice(CREDENTIALS.__dict__.keys())
+
 
 @click.group()
 def credentials():
+    """Get/set credentials."""
     pass
 
 
 def _get_setting(key):
-    try:
-        return getattr(CREDENTIALS, key)
-    except AttributeError as e:
-        raise ValueError(f"No such setting: {key}")
-
+    return getattr(CREDENTIALS, key)
 
 @credentials.command()
-@click.argument("key")
+def list():
+    """
+    List all credentials values.
+    """
+    output.print_spec(json.loads(CREDENTIALS.json(exclude_none=True)))
+
+@credentials.command()
+@click.argument("key", type=CHOICE_OF_CREDENTIAL_KEYS)
 def get(key):
     """
     Read a value from your credentials.
@@ -27,7 +35,7 @@ def get(key):
 
 
 @credentials.command()
-@click.argument("key")
+@click.argument("key", type=CHOICE_OF_CREDENTIAL_KEYS)
 @click.argument("value")
 def set(key, value):
     """
@@ -37,12 +45,17 @@ def set(key, value):
 
     current_credentials = _yaml_credentials_file_source(None)
     current_credentials[key] = value
+    try:
+        new_credentials = json.loads(CREDENTIALS(**current_credentials).json(exclude_none=True))
+    except pydantic.ValidationError as e:
+        errors = json.loads(e.json())
+        raise click.BadParameter("\n".join(error["msg"] for error in errors))
     with credentials_file().open("w") as f:
-        yaml.safe_dump(current_credentials, f)
+        yaml.safe_dump(new_credentials, f)
 
 
 @credentials.command()
-@click.argument("key")
+@click.argument("key", type=CHOICE_OF_CREDENTIAL_KEYS)
 def rm(key):
     """
     Delete a value from your credentials.
@@ -50,6 +63,6 @@ def rm(key):
     _get_setting(key)
 
     current_credentials = _yaml_credentials_file_source(None)
-    current_credentials.pop(key)
+    current_credentials.pop(key, None)
     with credentials_file().open("w") as f:
-        yaml.safe_dumps(current_credentials, f)
+        yaml.safe_dump(current_credentials, f)
